#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <errno.h>

#include "debug.h"
#include "rederrorcodes.h"
#include "controller.h"

const char *channel_names[] = { "dummy", "main", "display", "inputs", "cursor", "playback", "record" };

void QErrorHandler(int err, const char *custom_string)
{
    LOG_DEBUG("Something went wrong: " << custom_string << ", " << err);
}

SpiceController::SpiceController(MessageData &data)
{
    LOG_TRACE("");
    memcpy(&m_data, &data, sizeof(m_data));
}

SpiceController::~SpiceController()
{
    LOG_TRACE("");

    DisconnectClient();
}

int SpiceController::Connect()
{
    int nRC;

    LOG_TRACE("");
    struct sockaddr_un remote;

    if ((m_data.client_socket = socket (AF_UNIX, SOCK_STREAM, 0)) == -1) {
        QErrorHandler(errno, "SpiceController::Connect socket create error");
    }

    remote.sun_family = AF_UNIX;
    strcpy(remote.sun_path, m_data.name);

    nRC = connect(m_data.client_socket, (struct sockaddr *)&remote, strlen(remote.sun_path) + sizeof (remote.sun_family));
    if (nRC == -1) {
        QErrorHandler(errno, "connect error");
        LOG_DEBUG("Connect Error");
    } else {
        LOG_DEBUG("Connected!");
    }

    return (nRC);
}

int SpiceController::Connect(const int nRetries)
{
    int nRC;
    int nCount = 0;

    do {
        sleep(1);
        nRC = Connect();
        nCount++;
    } while (nRC != 0 && nCount < nRetries);

    return (nRC);
}

void SpiceController::DisconnectClient()
{
    LOG_TRACE("");
    close (m_data.client_socket);
    unlink(m_data.name);
}

void SpiceController::Write(const void *lpBuffer, uint32_t nBytesToWrite, uint32_t *nBytesWritten)
{
    LOG_TRACE("");
    int len;

    len = send (
        m_data.client_socket,
        lpBuffer,
        nBytesToWrite,
        0
    );

    if (len != nBytesToWrite) {
    LOG_WARN("send error, bytes to write = " << nBytesToWrite << ", bytes actually written = " << len << ", errno=" << errno);
        QErrorHandler(1, "send error");
    }

    *nBytesWritten = len;
}

int SpiceController::TranslateRC (int nRC) {
    LOG_TRACE("");
    int nReturn = 0;

    switch (nRC) {
    case SPICEC_ERROR_CODE_SUCCESS:
        nReturn = 0;
        break;
    case SPICEC_ERROR_CODE_GETHOSTBYNAME_FAILED:
		nReturn = RDP_ERROR_CODE_HOST_NOT_FOUND;
		break;
    case SPICEC_ERROR_CODE_CONNECT_FAILED:
		nReturn = RDP_ERROR_CODE_WINSOCK_CONNECT_FAILED;
		break;
    case SPICEC_ERROR_CODE_ERROR:
    case SPICEC_ERROR_CODE_SOCKET_FAILED:
		nReturn = RDP_ERROR_CODE_INTERNAL_ERROR;
		break;
    case SPICEC_ERROR_CODE_RECV_FAILED:
        nReturn = RDP_ERROR_RECV_WINSOCK_FAILED;
        break;
    case SPICEC_ERROR_CODE_SEND_FAILED:
        nReturn = RDP_ERROR_SEND_WINSOCK_FAILED;
        break;
    case SPICEC_ERROR_CODE_NOT_ENOUGH_MEMORY:
        nReturn = RDP_ERROR_CODE_OUT_OF_MEMORY;
        break;
    case SPICEC_ERROR_CODE_AGENT_TIMEOUT:
        nReturn = RDP_ERROR_CODE_TIMEOUT;
        break;
    case SPICEC_ERROR_CODE_AGENT_ERROR:
        nReturn = RDP_ERROR_CODE_INTERNAL_ERROR;
        break;
    default:
        nReturn = RDP_ERROR_CODE_INTERNAL_ERROR;
    }

    return nReturn;
}

