#ifndef __SpiceController_H
#define __SpiceController_H

/*
    Basic assumption:
    ------------------
    Cross platform compatible.
    Easy to transform into remote process communication
    Secured

    choosen:
        Unix - Unix Domain Sockets (easy to change into regular sockets for remote communication)
        Windows - Named pipe (which allows remote access and is duplex
            (rather than anonymous pipe which is local only and one way)
*/

#include <stdint.h>
#include <limits.h>
#include "controller_prot.h"

typedef struct MessageData {
    int client_socket;
    int nReadPos;
    int nCurrentReadLength;
    char name[PATH_MAX];
} MessageData;

class SpiceController {
public:
    SpiceController(MessageData &data);
    ~SpiceController();

    int Connect();
    int Connect(int nRetries);
    void DisconnectClient();
    void Write(const void *lpBuffer, uint32_t nBytesToWrite, uint32_t *nBytesWritten);
    int TranslateRC(int nRC);

private:
    MessageData m_data;
    wchar_t wcPipeName[256];
    uint32_t nOpenMode;
    uint32_t nPipeMode;
    uint32_t nMaxInstances;
    uint32_t nOutBufferSize;
    uint32_t nInBufferSize;
    bool _is_ipc_connect_recieved;
};

#endif
