#ifndef _H_DEBUG
#define _H_DEBUG

#include <sstream>

#include <log4cpp/Category.hh>
#include <log4cpp/convenience.h>

#define ON_PANIC() ::abort()

#ifdef RED_DEBUG

#ifdef WIN32
#define ASSERTBREAK DebugBreak()
#else
#define ASSERTBREAK ::abort()
#endif

#define ASSERT(x) if (!(x)) {                               \
    printf("%s: ASSERT %s failed\n", __FUNCTION__, #x);     \
    ASSERTBREAK;                                            \
}

#else

#define ASSERT(cond)

#endif

#ifdef __GNUC__
static inline std::string pretty_func_to_func_name(const std::string& f_name)
{
    std::string name(f_name);
    std::string::size_type end_pos = f_name.find('(');
    if (end_pos == std::string::npos) {
        return f_name;
    }
    std::string::size_type start = f_name.rfind(' ', end_pos);
    if (start == std::string::npos) {
        return f_name;
    }
    end_pos -= ++start;
    return name.substr(start, end_pos);
}
#define FUNC_NAME pretty_func_to_func_name(__PRETTY_FUNCTION__).c_str()
#else
#define FUNC_NAME __FUNCTION__
#endif

#define LOGGER_SECTION(section) LOG4CPP_LOGGER(section)

LOG4CPP_LOGGER("spice")

#define LOG(type, message) {                     \
    std::ostringstream os;   \
    os << FUNC_NAME;         \
    os << ": ";              \
    os << message;           \
    LOG4CPP_DEBUG(logger, os.str().c_str()); \
}

// Used to log just about everything that might be useful (or not).
#define LOG_TRACE(message) LOG(DEBUG, message)

// Used to log debug information which doesn't inflate the log with useless
// and repeated messages.
#define LOG_DEBUG(message) LOG(DEBUG, message)

// Used to log the program flow.
#define LOG_INFO(message) LOG(INFO, message)

// Used to log errors which are handled by the program.
#define LOG_WARN(message) LOG(WARN, message)

// Used to log errors that can't be handled by the program.
#define LOG_ERROR(message) LOG(ERROR, message)

// Used to log errors that can't happen (even if they do!).
#define LOG_FATAL(message) {     \
    LOG(EMERG, message);         \
    ON_PANIC();                  \
}

#if 0
#define DBGLEVEL 1000

#define DBG(level, msg) {       \
    if (level <= DBGLEVEL) {    \
        std::ostringstream os;  \
        os << __FUNCTION__;     \
        os << ": ";             \
        os << msg;              \
        os << "\n";             \
        std::cout << os.str();  \
        std::cout.flush();      \
    }                           \
}
#endif

#define PANIC(msg)      LOG_FATAL(msg)
#define WARN(msg)       LOG_WARN(msg)
#define ERR(msg)        LOG_ERROR(msg)
#define NOTIFY(msg)     LOG_INFO(msg)
#define INFO(msg)       LOG_INFO(msg)
#define DBG(level, msg) LOG_DEBUG(msg)

#endif // _H_DEBUG
