/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// ==============================
// ! Scriptability related code !
// ==============================

/////////////////////////////////////////////////////
//
// This file implements the nsScriptablePeer object
// The native methods of this class are supposed to
// be callable from JavaScript
//
#include <string.h>

#include "plugin.h"
#include "nsScriptablePeer.h"
#include "nsError.h"

/*
attribute string hostIP;
attribute string port;
attribute string SecurePort;
attribute string Password;
attribute string CipherSuite;
attribute string SSLChannels;
attribute boolean fullScreen;
attribute boolean AdminConsole;
attribute string Title;
attribute string dynamicMenu;
attribute string NumberOfMonitors;
attribute string GuestHostName;
attribute string HotKey;
attribute boolean NoTaskMgrExecution;
attribute boolean SendCtrlAltdelete;
attribute unsigned short UsbListenPort;
attribute boolean UsbAutoShare;

void connect();
void show();
void disconnect();
void SetLanguageStrings(in string section, in string lang);
void SetUsbFilter(in string filter);
long ConnectedStatus();
*/

static int NPIdentifiersSet = false;
static NPIdentifier hostIP_id;
static NPIdentifier port_id;
static NPIdentifier SecurePort_id;
static NPIdentifier Password_id;
static NPIdentifier CipherSuite_id;
static NPIdentifier SSLChannels_id;
static NPIdentifier TrustStore_id;
static NPIdentifier HostSubject_id;
static NPIdentifier fullScreen_id;
static NPIdentifier AdminConsole_id;
static NPIdentifier Title_id;
static NPIdentifier dynamicMenu_id;
static NPIdentifier NumberOfMonitors_id;
static NPIdentifier GuestHostName_id;
static NPIdentifier HotKey_id;
static NPIdentifier NoTaskMgrExecution_id;
static NPIdentifier SendCtrlAltdelete_id;
static NPIdentifier UsbListenPort_id;
static NPIdentifier UsbAutoShare_id;
static NPIdentifier connect_id;
static NPIdentifier show_id;
static NPIdentifier disconnect_id;
static NPIdentifier SetLanguageStrings_id;
static NPIdentifier SetUsbFilter_id;
static NPIdentifier ConnectedStatus_id;
static NPIdentifier PluginInstance_id;

void NPIdentifiersInit(void)
{  
  if(!NPIdentifiersSet) {
    hostIP_id = NPN_GetStringIdentifier("hostIP");
    port_id = NPN_GetStringIdentifier("port");
    SecurePort_id = NPN_GetStringIdentifier("SecurePort");
    Password_id = NPN_GetStringIdentifier("Password");
    CipherSuite_id = NPN_GetStringIdentifier("CipherSuite");
    SSLChannels_id = NPN_GetStringIdentifier("SSLChannels");
    TrustStore_id = NPN_GetStringIdentifier("TrustStore");
    HostSubject_id = NPN_GetStringIdentifier("HostSubject");
    fullScreen_id = NPN_GetStringIdentifier("fullScreen");
    AdminConsole_id = NPN_GetStringIdentifier("AdminConsole");
    Title_id = NPN_GetStringIdentifier("Title");
    dynamicMenu_id = NPN_GetStringIdentifier("dynamicMenu");
    NumberOfMonitors_id = NPN_GetStringIdentifier("NumberOfMonitors");
    GuestHostName_id = NPN_GetStringIdentifier("GuestHostName");
    HotKey_id = NPN_GetStringIdentifier("HotKey");
    NoTaskMgrExecution_id = NPN_GetStringIdentifier("NoTaskMgrExecution");
    SendCtrlAltdelete_id = NPN_GetStringIdentifier("SendCtrlAltdelete");
    UsbListenPort_id = NPN_GetStringIdentifier("UsbListenPort");
    UsbAutoShare_id = NPN_GetStringIdentifier("UsbAutoShare");
    connect_id = NPN_GetStringIdentifier("connect");
    show_id = NPN_GetStringIdentifier("show");
    disconnect_id = NPN_GetStringIdentifier("disconnect");
    SetLanguageStrings_id = NPN_GetStringIdentifier("SetLanguageStrings");
    SetUsbFilter_id = NPN_GetStringIdentifier("SetUsbFilter");
    ConnectedStatus_id = NPN_GetStringIdentifier("ConnectedStatus");
    PluginInstance_id = NPN_GetStringIdentifier("PluginInstance");
    NPIdentifiersSet = true;
  }
}

void
ScriptablePluginObjectBase::Invalidate()
{
}

bool
ScriptablePluginObjectBase::HasMethod(NPIdentifier name)
{
  return false;
}

bool
ScriptablePluginObjectBase::Invoke(NPIdentifier name, const NPVariant *args,
                                   uint32_t argCount, NPVariant *result)
{
  return false;
}

bool
ScriptablePluginObjectBase::InvokeDefault(const NPVariant *args,
                                          uint32_t argCount, NPVariant *result)
{
  return false;
}

bool
ScriptablePluginObjectBase::HasProperty(NPIdentifier name)
{
  return false;
}

bool
ScriptablePluginObjectBase::GetProperty(NPIdentifier name, NPVariant *result)
{
  return false;
}

bool
ScriptablePluginObjectBase::SetProperty(NPIdentifier name,
                                        const NPVariant *value)
{
  return false;
}

bool
ScriptablePluginObjectBase::RemoveProperty(NPIdentifier name)
{
  return false;
}

bool
ScriptablePluginObjectBase::Enumerate(NPIdentifier **identifier,
                                      uint32_t *count)
{
  return false;
}

bool
ScriptablePluginObjectBase::Construct(const NPVariant *args, uint32_t argCount,
                                      NPVariant *result)
{
  return false;
}

// static
void
ScriptablePluginObjectBase::_Deallocate(NPObject *npobj)
{
  // Call the virtual destructor.
  delete (ScriptablePluginObjectBase *)npobj;
}

// static
void
ScriptablePluginObjectBase::_Invalidate(NPObject *npobj)
{
  ((ScriptablePluginObjectBase *)npobj)->Invalidate();
}

// static
bool
ScriptablePluginObjectBase::_HasMethod(NPObject *npobj, NPIdentifier name)
{
  return ((ScriptablePluginObjectBase *)npobj)->HasMethod(name);
}

// static
bool
ScriptablePluginObjectBase::_Invoke(NPObject *npobj, NPIdentifier name,
                                    const NPVariant *args, uint32_t argCount,
                                    NPVariant *result)
{
  return ((ScriptablePluginObjectBase *)npobj)->Invoke(name, args, argCount,
                                                       result);
}

// static
bool
ScriptablePluginObjectBase::_InvokeDefault(NPObject *npobj,
                                           const NPVariant *args,
                                           uint32_t argCount,
                                           NPVariant *result)
{
  return ((ScriptablePluginObjectBase *)npobj)->InvokeDefault(args, argCount,
                                                              result);
}

// static
bool
ScriptablePluginObjectBase::_HasProperty(NPObject * npobj, NPIdentifier name)
{
  return ((ScriptablePluginObjectBase *)npobj)->HasProperty(name);
}

// static
bool
ScriptablePluginObjectBase::_GetProperty(NPObject *npobj, NPIdentifier name,
                                         NPVariant *result)
{
  return ((ScriptablePluginObjectBase *)npobj)->GetProperty(name, result);
}

// static
bool
ScriptablePluginObjectBase::_SetProperty(NPObject *npobj, NPIdentifier name,
                                         const NPVariant *value)
{
  return ((ScriptablePluginObjectBase *)npobj)->SetProperty(name, value);
}

// static
bool
ScriptablePluginObjectBase::_RemoveProperty(NPObject *npobj, NPIdentifier name)
{
  return ((ScriptablePluginObjectBase *)npobj)->RemoveProperty(name);
}

// static
bool
ScriptablePluginObjectBase::_Enumerate(NPObject *npobj,
                                       NPIdentifier **identifier,
                                       uint32_t *count)
{
  return ((ScriptablePluginObjectBase *)npobj)->Enumerate(identifier, count);
}

// static
bool
ScriptablePluginObjectBase::_Construct(NPObject *npobj, const NPVariant *args,
                                       uint32_t argCount, NPVariant *result)
{
  return ((ScriptablePluginObjectBase *)npobj)->Construct(args, argCount,
                                                          result);
}

NPObject *
AllocateScriptablePluginObject(NPP npp, NPClass *aClass)
{
  return new ScriptablePluginObject(npp);
}

bool
ScriptablePluginObject::HasMethod(NPIdentifier name)
{
  return( name == connect_id ||
          name == show_id ||
          name == disconnect_id ||
          name == SetLanguageStrings_id ||
          name == SetUsbFilter_id ||
          name == ConnectedStatus_id);
}

bool
ScriptablePluginObject::HasProperty(NPIdentifier name)
{
  return( name == hostIP_id ||
          name == port_id ||
          name == SecurePort_id ||
          name == Password_id ||
          name == CipherSuite_id ||
          name == SSLChannels_id ||
          name == TrustStore_id || 
          name == HostSubject_id ||
          name == fullScreen_id ||
          name == AdminConsole_id ||
          name == Title_id ||
          name == dynamicMenu_id ||
          name == NumberOfMonitors_id ||
          name == GuestHostName_id ||
          name == HotKey_id ||
          name == NoTaskMgrExecution_id ||
          name == SendCtrlAltdelete_id ||
          name == UsbListenPort_id ||
          name == UsbAutoShare_id);
}

bool
ScriptablePluginObject::GetProperty(NPIdentifier name, NPVariant *result)
{
  VOID_TO_NPVARIANT(*result);
  
  if (!mPlugin)
    return true;

  if (name == hostIP_id) {
    STRINGZ_TO_NPVARIANT(mPlugin->GetHostIP(), *result);
  }
  else if (name == port_id) {
    STRINGZ_TO_NPVARIANT(mPlugin->GetPort(), *result);
  }
  else if (name == SecurePort_id) {
    STRINGZ_TO_NPVARIANT(mPlugin->GetSecurePort(), *result);
  }
  else if (name == Password_id) {
    STRINGZ_TO_NPVARIANT(mPlugin->GetPassword(), *result);
  }
  else if (name == CipherSuite_id) {
    STRINGZ_TO_NPVARIANT(mPlugin->GetCipherSuite(), *result);
  }
  else if (name == SSLChannels_id) {
    STRINGZ_TO_NPVARIANT(mPlugin->GetSSLChannels(), *result);
  }
  else if (name == TrustStore_id) {
      STRINGZ_TO_NPVARIANT(mPlugin->GetTrustStore(), *result);
  }
  else if (name == HostSubject_id) {
      STRINGZ_TO_NPVARIANT(mPlugin->GetHostSubject(), *result);
  }
  else if (name == fullScreen_id) {
    BOOLEAN_TO_NPVARIANT(mPlugin->GetFullScreen(), *result);
  }
  else if (name == AdminConsole_id) {
    BOOLEAN_TO_NPVARIANT(mPlugin->GetAdminConsole(), *result);
  }
  else if (name == Title_id) {
    STRINGZ_TO_NPVARIANT(mPlugin->GetTitle(), *result);
  }
  else if (name == dynamicMenu_id) {
    STRINGZ_TO_NPVARIANT(mPlugin->GetDynamicMenu(), *result);
  }
  else if (name == NumberOfMonitors_id) {
    STRINGZ_TO_NPVARIANT(mPlugin->GetNumberOfMonitors(), *result);
  }
  else if (name == GuestHostName_id) {
    STRINGZ_TO_NPVARIANT(mPlugin->GetGuestHostName(), *result);
  }
  else if (name == HotKey_id) {
    STRINGZ_TO_NPVARIANT(mPlugin->GetHotKeys(), *result);
  }
  else if (name == NoTaskMgrExecution_id) {
    BOOLEAN_TO_NPVARIANT(mPlugin->GetNoTaskMgrExecution(), *result);
  }
  else if (name == SendCtrlAltdelete_id) {
    BOOLEAN_TO_NPVARIANT(mPlugin->GetSendCtrlAltdelete(), *result);
  }
  else if (name == UsbListenPort_id) {    
    INT32_TO_NPVARIANT(mPlugin->GetUsbListenPort(), *result);
  }
  else if (name == UsbAutoShare_id) {
    BOOLEAN_TO_NPVARIANT(mPlugin->GetUsbAutoShare(), *result);
  }
  else {
    return false;
  }

  return true;
}

bool
ScriptablePluginObject::SetProperty(NPIdentifier name, const NPVariant *value)
{
  if (!mPlugin)
    return false;

  const char *aString = NULL;  
  PRBool aBoolean = false;
  unsigned short aInt = -1;
  
  if (NPVARIANT_IS_STRING(*value)) {
    aString = NPVARIANT_TO_STRING(*value).UTF8Characters;
  } else if (NPVARIANT_IS_BOOLEAN(*value)) {
    aBoolean = NPVARIANT_TO_BOOLEAN(*value);
  } else if (NPVARIANT_IS_INT32(*value)) {
    aInt = NPVARIANT_TO_INT32(*value);
    static char tmp[200];
    snprintf(tmp,200,"%d",aInt);
    aString = tmp;
  } else {
    return false;
  }
  
  if (name == hostIP_id) {
    mPlugin->SetHostIP(aString);
  }
  else if (name == port_id) {
    mPlugin->SetPort(aString);
  }
  else if (name == SecurePort_id) {
    mPlugin->SetSecurePort(aString);
  }
  else if (name == Password_id) {
    mPlugin->SetPassword(aString);
  }
  else if (name == CipherSuite_id) {
    mPlugin->SetCipherSuite(aString);
  }
  else if (name == SSLChannels_id) {
    mPlugin->SetSSLChannels(aString);
  }
  else if (name == TrustStore_id) {
    mPlugin->SetTrustStore(aString);
  }
  else if (name == HostSubject_id) {
    mPlugin->SetHostSubject(aString);
  }
  else if (name == fullScreen_id) {
    mPlugin->SetFullScreen(aBoolean);
  }
  else if (name == AdminConsole_id) {
    mPlugin->SetAdminConsole(aBoolean);
  }
  else if (name == Title_id) {
    mPlugin->SetTitle(aString);
  }
  else if (name == dynamicMenu_id) {
    mPlugin->SetDynamicMenu(aString);
  }
  else if (name == NumberOfMonitors_id) {
    mPlugin->SetNumberOfMonitors(aString);
  }
  else if (name == GuestHostName_id) {
    mPlugin->SetGuestHostName(aString);
  }
  else if (name == HotKey_id) {
    mPlugin->SetHotKeys(aString);
  }
  else if (name == NoTaskMgrExecution_id) {
    mPlugin->SetNoTaskMgrExecution(aBoolean);
  }
  else if (name == SendCtrlAltdelete_id) {    
    mPlugin->SetSendCtrlAltdelete(aBoolean);  
  }
  else if (name == UsbListenPort_id) {
    mPlugin->SetUsbListenPort(aInt);
  }
  else if (name == UsbAutoShare_id) {
    mPlugin->SetUsbAutoShare(aBoolean);
  }
  else {
    return false;
  }

  return true;
}

bool
ScriptablePluginObject::Invoke(NPIdentifier name, const NPVariant *args,
                               uint32_t argCount, NPVariant *result)
{
  if (!mPlugin)
    return false;

  if (name == connect_id) {
    mPlugin->Connect();
    return true;
  }
  else if (name == show_id) {
    mPlugin->show();
    return true;
  }
  else if (name == disconnect_id) {
    mPlugin->disconnect();
    return true;
  }
  else if (name == SetLanguageStrings_id) {
    if(argCount < 2)
      return false;
    const char * aSection = NPVARIANT_TO_STRING(args[0]).UTF8Characters;
    const char * aLanguage = NPVARIANT_TO_STRING(args[1]).UTF8Characters;
    mPlugin->SetLanguageStrings(aSection, aLanguage);
    return true;
  }
  else if (name == SetUsbFilter_id) {
    if(argCount < 1)
      return false;
    const char * aUsbFilter = NPVARIANT_TO_STRING(args[0]).UTF8Characters;
    mPlugin->SetUsbFilter(aUsbFilter);
    return true;
  }
  else if (name == ConnectedStatus_id) {
    PRInt32 ret;
    mPlugin->ConnectedStatus(&ret);
    INT32_TO_NPVARIANT(ret, *result);
    return true;
  }   

  return false;
}

bool
ScriptablePluginObject::InvokeDefault(const NPVariant *args, uint32_t argCount,
                                      NPVariant *result)
{
  printf ("ScriptablePluginObject default method called!\n");
  return true;
}
